@@ -21,7 +21,9 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
 """
-from typing import Dict, List, Optional, Union
+from __future__ import annotations
+
+from typing import List
 
 import _discord
 from _discord.ext.bridge import BridgeContext
@@ -65,7 +67,7 @@ class PaginatorButton(_discord.ui.Button):
         self,
         button_type: str,
         label: str = None,
-        emoji: Union[str, _discord.Emoji, _discord.PartialEmoji] = None,
+        emoji: str | _discord.Emoji | _discord.PartialEmoji = None,
         style: _discord.ButtonStyle = _discord.ButtonStyle.green,
         disabled: bool = False,
         custom_id: str = None,
@@ -82,7 +84,7 @@ class PaginatorButton(_discord.ui.Button):
         )
         self.button_type = button_type
         self.label = label if label or emoji else button_type.capitalize()
-        self.emoji: Union[str, _discord.Emoji, _discord.PartialEmoji] = emoji
+        self.emoji: str | _discord.Emoji | _discord.PartialEmoji = emoji
         self.style = style
         self.disabled = disabled
         self.loop_label = self.label if not loop_label else loop_label
@@ -139,10 +141,10 @@ class Page:
 
     def __init__(
         self,
-        content: Optional[str] = None,
-        embeds: Optional[List[Union[List[_discord.Embed], _discord.Embed]]] = None,
-        custom_view: Optional[_discord.ui.View] = None,
-        files: Optional[List[_discord.File]] = None,
+        content: str | None = None,
+        embeds: list[list[_discord.Embed] | _discord.Embed] | None = None,
+        custom_view: _discord.ui.View | None = None,
+        files: list[_discord.File] | None = None,
         **kwargs,
     ):
         if content is None and embeds is None:
@@ -154,7 +156,7 @@ class Page:
         self._custom_view = custom_view
         self._files = files or []
 
-    async def callback(self, interaction: Optional[_discord.Interaction] = None):
+    async def callback(self, interaction: _discord.Interaction | None = None):
         """|coro|
 
         The coroutine associated to a specific page. If `Paginator.page_action()` is used, this coroutine is called.
@@ -165,57 +167,54 @@ class Page:
             The interaction associated with the callback, if any.
         """
 
-    def update_files(self) -> Optional[List[_discord.File]]:
-        """Updates the files associated with the page by re-uploading them.
-        Typically used when the page is changed.
+    def update_files(self) -> list[_discord.File] | None:
+        """Updates :class:`discord.File` objects so that they can be sent multiple
+        times. This is called internally each time the page is sent.
         """
         for file in self._files:
-            with open(file.fp.name, "rb") as fp:  # type: ignore
-                self._files[self._files.index(file)] = _discord.File(
-                    fp,  # type: ignore
-                    filename=file.filename,
-                    description=file.description,
-                    spoiler=file.spoiler,
-                )
+            if file.fp.closed and (fn := getattr(file.fp, "name", None)):
+                file.fp = open(fn, "rb")
+            file.reset()
+            file.fp.close = lambda: None
         return self._files
 
     @property
-    def content(self) -> Optional[str]:
+    def content(self) -> str | None:
         """Gets the content for the page."""
         return self._content
 
     @content.setter
-    def content(self, value: Optional[str]):
+    def content(self, value: str | None):
         """Sets the content for the page."""
         self._content = value
 
     @property
-    def embeds(self) -> Optional[List[Union[List[_discord.Embed], _discord.Embed]]]:
+    def embeds(self) -> list[list[_discord.Embed] | _discord.Embed] | None:
         """Gets the embeds for the page."""
         return self._embeds
 
     @embeds.setter
-    def embeds(self, value: Optional[List[Union[List[_discord.Embed], _discord.Embed]]]):
+    def embeds(self, value: list[list[_discord.Embed] | _discord.Embed] | None):
         """Sets the embeds for the page."""
         self._embeds = value
 
     @property
-    def custom_view(self) -> Optional[_discord.ui.View]:
+    def custom_view(self) -> _discord.ui.View | None:
         """Gets the custom view assigned to the page."""
         return self._custom_view
 
     @custom_view.setter
-    def custom_view(self, value: Optional[_discord.ui.View]):
+    def custom_view(self, value: _discord.ui.View | None):
         """Assigns a custom view to be shown when the page is displayed."""
         self._custom_view = value
 
     @property
-    def files(self) -> Optional[List[_discord.File]]:
+    def files(self) -> list[_discord.File] | None:
         """Gets the files associated with the page."""
         return self._files
 
     @files.setter
-    def files(self, value: Optional[List[_discord.File]]):
+    def files(self, value: list[_discord.File] | None):
         """Sets the files associated with the page."""
         self._files = value
 
@@ -272,32 +271,28 @@ class PageGroup:
 
     def __init__(
         self,
-        pages: Union[
-            List[str], List[Page], List[Union[List[_discord.Embed], _discord.Embed]]
-        ],
+        pages: (list[str] | list[Page] | list[list[_discord.Embed] | _discord.Embed]),
         label: str,
-        description: Optional[str] = None,
-        emoji: Union[str, _discord.Emoji, _discord.PartialEmoji] = None,
-        default: Optional[bool] = None,
-        show_disabled: Optional[bool] = None,
-        show_indicator: Optional[bool] = None,
-        author_check: Optional[bool] = None,
-        disable_on_timeout: Optional[bool] = None,
-        use_default_buttons: Optional[bool] = None,
+        description: str | None = None,
+        emoji: str | _discord.Emoji | _discord.PartialEmoji = None,
+        default: bool | None = None,
+        show_disabled: bool | None = None,
+        show_indicator: bool | None = None,
+        author_check: bool | None = None,
+        disable_on_timeout: bool | None = None,
+        use_default_buttons: bool | None = None,
         default_button_row: int = 0,
-        loop_pages: Optional[bool] = None,
-        custom_view: Optional[_discord.ui.View] = None,
-        timeout: Optional[float] = None,
-        custom_buttons: Optional[List[PaginatorButton]] = None,
-        trigger_on_display: Optional[bool] = None,
+        loop_pages: bool | None = None,
+        custom_view: _discord.ui.View | None = None,
+        timeout: float | None = None,
+        custom_buttons: list[PaginatorButton] | None = None,
+        trigger_on_display: bool | None = None,
     ):
         self.label = label
-        self.description: Optional[str] = description
-        self.emoji: Union[str, _discord.Emoji, _discord.PartialEmoji] = emoji
-        self.pages: Union[
-            List[str], List[Union[List[_discord.Embed], _discord.Embed]]
-        ] = pages
-        self.default: Optional[bool] = default
+        self.description: str | None = description
+        self.emoji: str | _discord.Emoji | _discord.PartialEmoji = emoji
+        self.pages: (list[str] | list[list[_discord.Embed] | _discord.Embed]) = pages
+        self.default: bool | None = default
         self.show_disabled = show_disabled
         self.show_indicator = show_indicator
         self.author_check = author_check
@@ -307,7 +302,7 @@ class PageGroup:
         self.loop_pages = loop_pages
         self.custom_view: _discord.ui.View = custom_view
         self.timeout: float = timeout
-        self.custom_buttons: List = custom_buttons
+        self.custom_buttons: list = custom_buttons
         self.trigger_on_display = trigger_on_display
 
 
@@ -375,12 +370,12 @@ class Paginator(_discord.ui.View):
 
     def __init__(
         self,
-        pages: Union[
-            List[PageGroup],
-            List[Page],
-            List[str],
-            List[Union[List[_discord.Embed], _discord.Embed]],
-        ],
+        pages: (
+            list[PageGroup]
+            | list[Page]
+            | list[str]
+            | list[list[_discord.Embed] | _discord.Embed]
+        ),
         show_disabled: bool = True,
         show_indicator=True,
         show_menu=False,
@@ -390,24 +385,24 @@ class Paginator(_discord.ui.View):
         use_default_buttons=True,
         default_button_row: int = 0,
         loop_pages=False,
-        custom_view: Optional[_discord.ui.View] = None,
-        timeout: Optional[float] = 180.0,
-        custom_buttons: Optional[List[PaginatorButton]] = None,
-        trigger_on_display: Optional[bool] = None,
+        custom_view: _discord.ui.View | None = None,
+        timeout: float | None = 180.0,
+        custom_buttons: list[PaginatorButton] | None = None,
+        trigger_on_display: bool | None = None,
     ) -> None:
         super().__init__(timeout=timeout)
         self.timeout: float = timeout
-        self.pages: Union[
-            List[PageGroup],
-            List[str],
-            List[Page],
-            List[Union[List[_discord.Embed], _discord.Embed]],
-        ] = pages
+        self.pages: (
+            list[PageGroup]
+            | list[str]
+            | list[Page]
+            | list[list[_discord.Embed] | _discord.Embed]
+        ) = pages
         self.current_page = 0
-        self.menu: Optional[PaginatorMenu] = None
+        self.menu: PaginatorMenu | None = None
         self.show_menu = show_menu
         self.menu_placeholder = menu_placeholder
-        self.page_groups: Optional[List[PageGroup]] = None
+        self.page_groups: list[PageGroup] | None = None
         self.default_page_group: int = 0
 
         if all(isinstance(pg, PageGroup) for pg in pages):
@@ -418,13 +413,13 @@ class Paginator(_discord.ui.View):
                 if pg.default:
                     self.default_page_group = self.page_groups.index(pg)
                     break
-            self.pages: List[Page] = self.get_page_group_content(
+            self.pages: list[Page] = self.get_page_group_content(
                 self.page_groups[self.default_page_group]
             )
 
         self.page_count = max(len(self.pages) - 1, 0)
         self.buttons = {}
-        self.custom_buttons: List = custom_buttons
+        self.custom_buttons: list = custom_buttons
         self.show_disabled = show_disabled
         self.show_indicator = show_indicator
         self.disable_on_timeout = disable_on_timeout
@@ -433,7 +428,7 @@ class Paginator(_discord.ui.View):
         self.loop_pages = loop_pages
         self.custom_view: _discord.ui.View = custom_view
         self.trigger_on_display = trigger_on_display
-        self.message: Union[_discord.Message, _discord.WebhookMessage, None] = None
+        self.message: _discord.Message | _discord.WebhookMessage | None = None
 
         if self.custom_buttons and not self.use_default_buttons:
             for button in custom_buttons:
@@ -449,28 +444,27 @@ class Paginator(_discord.ui.View):
 
     async def update(
         self,
-        pages: Optional[
-            Union[
-                List[PageGroup],
-                List[Page],
-                List[str],
-                List[Union[List[_discord.Embed], _discord.Embed]],
-            ]
-        ] = None,
-        show_disabled: Optional[bool] = None,
-        show_indicator: Optional[bool] = None,
-        show_menu: Optional[bool] = None,
-        author_check: Optional[bool] = None,
-        menu_placeholder: Optional[str] = None,
-        disable_on_timeout: Optional[bool] = None,
-        use_default_buttons: Optional[bool] = None,
-        default_button_row: Optional[int] = None,
-        loop_pages: Optional[bool] = None,
-        custom_view: Optional[_discord.ui.View] = None,
-        timeout: Optional[float] = None,
-        custom_buttons: Optional[List[PaginatorButton]] = None,
-        trigger_on_display: Optional[bool] = None,
-        interaction: Optional[_discord.Interaction] = None,
+        pages: None
+        | (
+            list[PageGroup]
+            | list[Page]
+            | list[str]
+            | list[list[_discord.Embed] | _discord.Embed]
+        ) = None,
+        show_disabled: bool | None = None,
+        show_indicator: bool | None = None,
+        show_menu: bool | None = None,
+        author_check: bool | None = None,
+        menu_placeholder: str | None = None,
+        disable_on_timeout: bool | None = None,
+        use_default_buttons: bool | None = None,
+        default_button_row: int | None = None,
+        loop_pages: bool | None = None,
+        custom_view: _discord.ui.View | None = None,
+        timeout: float | None = None,
+        custom_buttons: list[PaginatorButton] | None = None,
+        trigger_on_display: bool | None = None,
+        interaction: _discord.Interaction | None = None,
     ):
         """Updates the existing :class:`Paginator` instance with the provided options.
 
@@ -514,14 +508,12 @@ class Paginator(_discord.ui.View):
         """
 
         # Update pages and reset current_page to 0 (default)
-        self.pages: Union[
-            List[PageGroup],
-            List[str],
-            List[Page],
-            List[Union[List[_discord.Embed], _discord.Embed]],
-        ] = (
-            pages if pages is not None else self.pages
-        )
+        self.pages: (
+            list[PageGroup]
+            | list[str]
+            | list[Page]
+            | list[list[_discord.Embed] | _discord.Embed]
+        ) = (pages if pages is not None else self.pages)
         self.show_menu = show_menu if show_menu is not None else self.show_menu
         if pages is not None and all(isinstance(pg, PageGroup) for pg in pages):
             self.page_groups = self.pages if self.show_menu else None
@@ -531,7 +523,7 @@ class Paginator(_discord.ui.View):
                 if pg.default:
                     self.default_page_group = self.page_groups.index(pg)
                     break
-            self.pages: List[Page] = self.get_page_group_content(
+            self.pages: list[Page] = self.get_page_group_content(
                 self.page_groups[self.default_page_group]
             )
         self.page_count = max(len(self.pages) - 1, 0)
@@ -597,9 +589,7 @@ class Paginator(_discord.ui.View):
     async def disable(
         self,
         include_custom: bool = False,
-        page: Optional[
-            Union[str, Page, Union[List[_discord.Embed], _discord.Embed]]
-        ] = None,
+        page: None | (str | Page | list[_discord.Embed] | _discord.Embed) = None,
     ) -> None:
         """Stops the paginator, disabling all of its components.
 
@@ -630,9 +620,7 @@ class Paginator(_discord.ui.View):
     async def cancel(
         self,
         include_custom: bool = False,
-        page: Optional[
-            Union[str, Page, Union[List[_discord.Embed], _discord.Embed]]
-        ] = None,
+        page: None | (str | Page | list[_discord.Embed] | _discord.Embed) = None,
     ) -> None:
         """Cancels the paginator, removing all of its components from the message.
 
@@ -662,7 +650,7 @@ class Paginator(_discord.ui.View):
             await self.message.edit(view=self)
 
     async def goto_page(
-        self, page_number: int = 0, *, interaction: Optional[_discord.Interaction] = None
+        self, page_number: int = 0, *, interaction: _discord.Interaction | None = None
     ) -> None:
         """Updates the paginator message to show the specified page number.
 
@@ -778,11 +766,15 @@ class Paginator(_discord.ui.View):
         self.buttons[button.button_type] = {
             "object": _discord.ui.Button(
                 style=button.style,
-                label=button.label
-                if button.label or button.emoji
-                else button.button_type.capitalize()
-                if button.button_type != "page_indicator"
-                else f"{self.current_page + 1}/{self.page_count + 1}",
+                label=(
+                    button.label
+                    if button.label or button.emoji
+                    else (
+                        button.button_type.capitalize()
+                        if button.button_type != "page_indicator"
+                        else f"{self.current_page + 1}/{self.page_count + 1}"
+                    )
+                ),
                 disabled=button.disabled,
                 custom_id=button.custom_id,
                 emoji=button.emoji,
@@ -790,9 +782,11 @@ class Paginator(_discord.ui.View):
             ),
             "label": button.label,
             "loop_label": button.loop_label,
-            "hidden": button.disabled
-            if button.button_type != "page_indicator"
-            else not self.show_indicator,
+            "hidden": (
+                button.disabled
+                if button.button_type != "page_indicator"
+                else not self.show_indicator
+            ),
         }
         self.buttons[button.button_type]["object"].callback = button.callback
         button.paginator = self
@@ -805,7 +799,7 @@ class Paginator(_discord.ui.View):
             )
         self.buttons.pop(button_type)
 
-    def update_buttons(self) -> Dict:
+    def update_buttons(self) -> dict:
         """Updates the display state of the buttons (disabled/hidden)
 
         Returns
@@ -879,13 +873,13 @@ class Paginator(_discord.ui.View):
         for item in custom_view.children:
             self.add_item(item)
 
-    def get_page_group_content(self, page_group: PageGroup) -> List[Page]:
+    def get_page_group_content(self, page_group: PageGroup) -> list[Page]:
         """Returns a converted list of `Page` objects for the given page group based on the content of its pages."""
         return [self.get_page_content(page) for page in page_group.pages]
 
     @staticmethod
     def get_page_content(
-        page: Union[Page, str, _discord.Embed, List[_discord.Embed]]
+        page: Page | str | _discord.Embed | list[_discord.Embed],
     ) -> Page:
         """Converts a page into a :class:`Page` object based on its content."""
         if isinstance(page, Page):
@@ -909,9 +903,7 @@ class Paginator(_discord.ui.View):
                 " embeds, a file, or a list of files."
             )
 
-    async def page_action(
-        self, interaction: Optional[_discord.Interaction] = None
-    ) -> None:
+    async def page_action(self, interaction: _discord.Interaction | None = None) -> None:
         """Triggers the callback associated with the current page, if any.
 
         Parameters
@@ -927,14 +919,13 @@ class Paginator(_discord.ui.View):
     async def send(
         self,
         ctx: Context,
-        target: Optional[_discord.abc.Messageable] = None,
-        target_message: Optional[str] = None,
-        reference: Optional[
-            Union[_discord.Message, _discord.MessageReference, _discord.PartialMessage]
-        ] = None,
-        allowed_mentions: Optional[_discord.AllowedMentions] = None,
-        mention_author: Optional[bool] = None,
-        delete_after: Optional[float] = None,
+        target: _discord.abc.Messageable | None = None,
+        target_message: str | None = None,
+        reference: None
+        | (_discord.Message | _discord.MessageReference | _discord.PartialMessage) = None,
+        allowed_mentions: _discord.AllowedMentions | None = None,
+        mention_author: bool | None = None,
+        delete_after: float | None = None,
     ) -> _discord.Message:
         """Sends a message with the paginated items.
 
@@ -1029,10 +1020,10 @@ class Paginator(_discord.ui.View):
     async def edit(
         self,
         message: _discord.Message,
-        suppress: Optional[bool] = None,
-        allowed_mentions: Optional[_discord.AllowedMentions] = None,
-        delete_after: Optional[float] = None,
-    ) -> Optional[_discord.Message]:
+        suppress: bool | None = None,
+        allowed_mentions: _discord.AllowedMentions | None = None,
+        delete_after: float | None = None,
+    ) -> _discord.Message | None:
         """Edits an existing message to replace it with the paginator contents.
 
         .. note::
@@ -1068,7 +1059,7 @@ class Paginator(_discord.ui.View):
 
         self.update_buttons()
 
-        page: Union[Page, str, _discord.Embed, List[_discord.Embed]] = self.pages[
+        page: Page | str | _discord.Embed | list[_discord.Embed] = self.pages[
             self.current_page
         ]
         page_content: Page = self.get_page_content(page)
@@ -1096,11 +1087,11 @@ class Paginator(_discord.ui.View):
 
     async def respond(
         self,
-        interaction: Union[_discord.Interaction, BridgeContext],
+        interaction: _discord.Interaction | BridgeContext,
         ephemeral: bool = False,
-        target: Optional[_discord.abc.Messageable] = None,
+        target: _discord.abc.Messageable | None = None,
         target_message: str = "Paginator sent!",
-    ) -> Union[_discord.Message, _discord.WebhookMessage]:
+    ) -> _discord.Message | _discord.WebhookMessage:
         """Sends an interaction response or followup with the paginated items.
 
         Parameters
@@ -1145,7 +1136,7 @@ class Paginator(_discord.ui.View):
 
         self.update_buttons()
 
-        page: Union[Page, str, _discord.Embed, List[_discord.Embed]] = self.pages[
+        page: Page | str | _discord.Embed | list[_discord.Embed] = self.pages[
             self.current_page
         ]
         page_content: Page = self.get_page_content(page)
@@ -1229,12 +1220,12 @@ class PaginatorMenu(_discord.ui.Select):
 
     def __init__(
         self,
-        page_groups: List[PageGroup],
-        placeholder: Optional[str] = None,
-        custom_id: Optional[str] = None,
+        page_groups: list[PageGroup],
+        placeholder: str | None = None,
+        custom_id: str | None = None,
     ):
         self.page_groups = page_groups
-        self.paginator: Optional[Paginator] = None
+        self.paginator: Paginator | None = None
         opts = [
             _discord.SelectOption(
                 label=page_group.label,
