@@ -554,8 +554,10 @@ class ConnectionState:
             return await asyncio.wait_for(request.wait(), timeout=30.0)
         except asyncio.TimeoutError:
             _log.warning(
-                "Timed out waiting for chunks with query %r and limit %d for"
-                " guild_id %d",
+                (
+                    "Timed out waiting for chunks with query %r and limit %d for"
+                    " guild_id %d"
+                ),
                 query,
                 limit,
                 guild_id,
@@ -926,8 +928,10 @@ class ConnectionState:
 
         if channel is None:
             _log.debug(
-                "CHANNEL_PINS_UPDATE referencing an unknown channel ID: %s."
-                " Discarding.",
+                (
+                    "CHANNEL_PINS_UPDATE referencing an unknown channel ID: %s."
+                    " Discarding."
+                ),
                 channel_id,
             )
             return
@@ -965,23 +969,28 @@ class ConnectionState:
     def parse_thread_update(self, data) -> None:
         guild_id = int(data["guild_id"])
         guild = self._get_guild(guild_id)
+        raw = RawThreadUpdateEvent(data)
         if guild is None:
             _log.debug(
                 "THREAD_UPDATE referencing an unknown guild ID: %s. Discarding",
                 guild_id,
             )
             return
-
-        thread_id = int(data["id"])
-        thread = guild.get_thread(thread_id)
-        if thread is not None:
-            old = copy.copy(thread)
-            thread._update(data)
-            self.dispatch("thread_update", old, thread)
         else:
-            thread = Thread(guild=guild, state=guild._state, data=data)
-            guild._add_thread(thread)
-            self.dispatch("thread_join", thread)
+            thread = guild.get_thread(raw.thread_id)
+            if thread is not None:
+                old = copy.copy(thread)
+                thread._update(data)
+                if thread.archived:
+                    guild._remove_thread(thread)
+                self.dispatch("thread_update", old, thread)
+            else:
+                thread = Thread(guild=guild, state=guild._state, data=data)
+                if not thread.archived:
+                    guild._add_thread(thread)
+                self.dispatch("thread_join", thread)
+            raw.thread = thread
+        self.dispatch("raw_thread_update", raw)
 
     def parse_thread_delete(self, data) -> None:
         guild_id = int(data["guild_id"])
@@ -1080,10 +1089,13 @@ class ConnectionState:
 
         thread_id = int(data["id"])
         thread: Thread | None = guild.get_thread(thread_id)
+        raw = RawThreadMembersUpdateEvent(data)
         if thread is None:
             _log.debug(
-                "THREAD_MEMBERS_UPDATE referencing an unknown thread ID: %s."
-                " Discarding",
+                (
+                    "THREAD_MEMBERS_UPDATE referencing an unknown thread ID: %s."
+                    " Discarding"
+                ),
                 thread_id,
             )
             return
@@ -1102,6 +1114,7 @@ class ConnectionState:
         for member_id in removed_member_ids:
             if member_id != self_id:
                 member = thread._pop_member(member_id)
+                self.dispatch("raw_thread_member_remove", raw)
                 if member is not None:
                     self.dispatch("thread_member_remove", member)
             else:
@@ -1128,6 +1141,9 @@ class ConnectionState:
         self.dispatch("member_join", member)
 
     def parse_guild_member_remove(self, data) -> None:
+        user = self.store_user(data["user"])
+        raw = RawMemberRemoveEvent(data, user)
+
         guild = self._get_guild(int(data["guild_id"]))
         if guild is not None:
             try:
@@ -1135,9 +1151,9 @@ class ConnectionState:
             except AttributeError:
                 pass
 
-            user_id = int(data["user"]["id"])
-            member = guild.get_member(user_id)
+            member = guild.get_member(user.id)
             if member is not None:
+                raw.user = member
                 guild._remove_member(member)  # type: ignore
                 self.dispatch("member_remove", member)
         else:
@@ -1145,6 +1161,7 @@ class ConnectionState:
                 "GUILD_MEMBER_REMOVE referencing an unknown guild ID: %s. Discarding.",
                 data["guild_id"],
             )
+        self.dispatch("raw_member_remove", raw)
 
     def parse_guild_member_update(self, data) -> None:
         guild = self._get_guild(int(data["guild_id"]))
@@ -1201,8 +1218,10 @@ class ConnectionState:
         guild = self._get_guild(int(data["guild_id"]))
         if guild is None:
             _log.debug(
-                "GUILD_STICKERS_UPDATE referencing an unknown guild ID: %s."
-                " Discarding.",
+                (
+                    "GUILD_STICKERS_UPDATE referencing an unknown guild ID: %s."
+                    " Discarding."
+                ),
                 data["guild_id"],
             )
             return
@@ -1415,8 +1434,10 @@ class ConnectionState:
         guild = self._get_guild(int(data["guild_id"]))
         if guild is None:
             _log.debug(
-                "GUILD_SCHEDULED_EVENT_CREATE referencing an unknown guild ID: %s."
-                " Discarding.",
+                (
+                    "GUILD_SCHEDULED_EVENT_CREATE referencing an unknown guild ID: %s."
+                    " Discarding."
+                ),
                 data["guild_id"],
             )
             return
@@ -1436,8 +1457,10 @@ class ConnectionState:
         guild = self._get_guild(int(data["guild_id"]))
         if guild is None:
             _log.debug(
-                "GUILD_SCHEDULED_EVENT_UPDATE referencing an unknown guild ID: %s."
-                " Discarding.",
+                (
+                    "GUILD_SCHEDULED_EVENT_UPDATE referencing an unknown guild ID: %s."
+                    " Discarding."
+                ),
                 data["guild_id"],
             )
             return
@@ -1458,8 +1481,10 @@ class ConnectionState:
         guild = self._get_guild(int(data["guild_id"]))
         if guild is None:
             _log.debug(
-                "GUILD_SCHEDULED_EVENT_DELETE referencing an unknown guild ID: %s."
-                " Discarding.",
+                (
+                    "GUILD_SCHEDULED_EVENT_DELETE referencing an unknown guild ID: %s."
+                    " Discarding."
+                ),
                 data["guild_id"],
             )
             return
@@ -1480,8 +1505,10 @@ class ConnectionState:
         guild = self._get_guild(int(data["guild_id"]))
         if guild is None:
             _log.debug(
-                "GUILD_SCHEDULED_EVENT_USER_ADD referencing an unknown guild ID: %s."
-                " Discarding.",
+                (
+                    "GUILD_SCHEDULED_EVENT_USER_ADD referencing an unknown guild ID:"
+                    " %s. Discarding."
+                ),
                 data["guild_id"],
             )
             return
@@ -1502,8 +1529,10 @@ class ConnectionState:
         guild = self._get_guild(int(data["guild_id"]))
         if guild is None:
             _log.debug(
-                "GUILD_SCHEDULED_EVENT_USER_REMOVE referencing an unknown guild ID: %s."
-                " Discarding.",
+                (
+                    "GUILD_SCHEDULED_EVENT_USER_REMOVE referencing an unknown guild ID:"
+                    " %s. Discarding."
+                ),
                 data["guild_id"],
             )
             return
@@ -1526,8 +1555,10 @@ class ConnectionState:
             self.dispatch("guild_integrations_update", guild)
         else:
             _log.debug(
-                "GUILD_INTEGRATIONS_UPDATE referencing an unknown guild ID: %s."
-                " Discarding.",
+                (
+                    "GUILD_INTEGRATIONS_UPDATE referencing an unknown guild ID: %s."
+                    " Discarding."
+                ),
                 data["guild_id"],
             )
 
@@ -1611,8 +1642,10 @@ class ConnectionState:
                 )
             else:
                 _log.debug(
-                    "STAGE_INSTANCE_UPDATE referencing unknown stage instance ID: %s."
-                    " Discarding.",
+                    (
+                        "STAGE_INSTANCE_UPDATE referencing unknown stage instance ID:"
+                        " %s. Discarding."
+                    ),
                     data["id"],
                 )
         else:
@@ -1669,8 +1702,10 @@ class ConnectionState:
                 self.dispatch("voice_state_update", member, before, after)
             else:
                 _log.debug(
-                    "VOICE_STATE_UPDATE referencing an unknown member ID: %s."
-                    " Discarding.",
+                    (
+                        "VOICE_STATE_UPDATE referencing an unknown member ID: %s."
+                        " Discarding."
+                    ),
                     data["user_id"],
                 )
 
@@ -1831,8 +1866,10 @@ class AutoShardedConnectionState(ConnectionState):
             else:
                 if self._guild_needs_chunking(guild):
                     _log.debug(
-                        "Guild ID %d requires chunking, will be done in the"
-                        " background.",
+                        (
+                            "Guild ID %d requires chunking, will be done in the"
+                            " background."
+                        ),
                         guild.id,
                     )
                     if len(current_bucket) >= max_concurrency:
@@ -1867,8 +1904,10 @@ class AutoShardedConnectionState(ConnectionState):
                 await utils.sane_wait_for(futures, timeout=timeout)
             except asyncio.TimeoutError:
                 _log.warning(
-                    "Shard ID %s failed to wait for chunks (timeout=%.2f) for %d"
-                    " guilds",
+                    (
+                        "Shard ID %s failed to wait for chunks (timeout=%.2f) for %d"
+                        " guilds"
+                    ),
                     shard_id,
                     timeout,
                     len(guilds),
