@@ -1,4 +1,4 @@
 "Package used for logging"
-from .logging import *
+from ._logging import *
 from .sql import *
 from .tracing import *
