@@ -7,14 +7,14 @@
     .. versionchanged:: v2.1
         Made SQL an optional functionality
 """
-from datetime import datetime
+from datetime import datetime, date
 from typing import Callable, Dict, List, Literal, Any, Union, Optional
 from contextlib import suppress
 from typeguard import typechecked
 
 from .tracing import TraceLEVELS, trace
 
-from . import logging
+from . import _logging as logging
 
 import json
 import copy
@@ -43,12 +43,11 @@ DIALECT_CONN_MAP = {
     "postgresql": "asyncpg",
     "mysql": "asyncmy"
 }
-
 # ------------------------------------ Optional ------------------------------------
 try:
     from sqlalchemy import (
         SmallInteger, Integer, BigInteger, DateTime,
-        Sequence, String, JSON, select, text, ForeignKey
+        Sequence, String, JSON, select, text, ForeignKey, func, case
     )
     from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
     from sqlalchemy.engine import URL as SQLURL, create_engine
@@ -58,6 +57,7 @@ try:
         Session,
         DeclarativeBase,
         mapped_column,
+        column_property,
         relationship,
         Mapped,
     )
@@ -117,6 +117,19 @@ def register_type(lookuptable: Literal["GuildTYPE", "MessageTYPE", "MessageMODE"
 class ORMBase(DeclarativeBase):
     "Base for all of ORM classes"
 
+    def __eq__(self, value: object) -> bool:
+        try:
+            return self.id == value.id and type(self) is type(value)
+        except AttributeError:
+            return super().__eq__(value)
+
+    def __hash__(self):
+        try:
+            return self.id << 4 | id(type(self)) >> 8
+        except AttributeError:
+            return super().__hash__()
+
+
 
 class TableCache:
     """
@@ -250,7 +263,7 @@ class LoggerSQL(logging.LoggerBASE):
                  server: Optional[str] = None,
                  port: Optional[int] = None,
                  database: Optional[str] = None,
-                 dialect: Optional[str] = None,
+                 dialect: Literal["sqlite", "mssql", "postgresql", "mysql"] = None,
                  fallback: Optional[logging.LoggerBASE] = ...):
 
         if not SQL_INSTALLED:
@@ -695,16 +708,19 @@ class LoggerSQL(logging.LoggerBASE):
     @misc._async_safe("safe_sem", 1)
     async def _save_log(self,
                         guild_context: dict,
-                        message_context: dict):
+                        message_context: dict,
+                        author_context: dict):
         """
         This method saves the log generated by the xGUILD object into the database.
 
         Parameters
         -------------
         guild_context: dict
-            Context generated by the xGUILD object, see guild.xGUILD._generate_log() for more info.
+            Context generated by the xGUILD object, see guild.xGUILD.generate_log_context() for more info.
         message_context: dict
             Context generated by the xMESSAGE object, see guild.xMESSAGE.generate_log_context() for more info.
+        author_context: dict
+            Context generated by the ACCOUNT object, see ACCOUNT.generate_log_context() for more info.
 
         Raises
         --------
@@ -720,6 +736,8 @@ class LoggerSQL(logging.LoggerBASE):
             return
 
         # Parse the data
+        author_name: str = author_context.get("name")
+        author_snowflake: str = author_context.get("id")
         sent_data: dict = message_context.get("sent_data")
         guild_snowflake: int = guild_context.get("id")
         guild_name: str = guild_context.get("name")
@@ -751,6 +769,12 @@ class LoggerSQL(logging.LoggerBASE):
                         _channels = await self._get_insert_channels(channels, guild_obj, session)
 
                     data_obj = await self._get_insert_data(sent_data, session)
+                    author_obj = await self._get_insert_guild(
+                        author_snowflake,
+                        author_name,
+                        self.guild_type_cache.get("USER"),
+                        session
+                    )
 
                     # Save message log
                     message_log_obj = MessageLOG(
@@ -759,6 +783,7 @@ class LoggerSQL(logging.LoggerBASE):
                         message_mode_obj,
                         dm_success_info_reason,
                         guild_obj,
+                        author_obj,
                         [
                             MessageChannelLOG(channel, reason)
                             for channel, reason in _channels
@@ -776,6 +801,239 @@ class LoggerSQL(logging.LoggerBASE):
         else:
             raise RuntimeError(f"Unable to save log within {SQL_MAX_SAVE_ATTEMPTS} tries")
 
+    async def _get_guild(self, id_: int, session: AsyncSession | Session):
+        guilduser: GuildUSER = self.guild_user_cache.get(id_)
+        if guilduser is not None:
+            return guilduser
+
+        try:
+            _ret = (await self._run_async(
+                session.execute,
+                select(GuildUSER)
+                .where(GuildUSER.snowflake_id == id_))
+            ).first()
+            return _ret[0] if _ret is not None else None
+        except SQLAlchemyError:
+            return None
+
+    async def analytic_get_num_messages(
+            self,
+            guild: int | None = None,
+            author: int | None = None,
+            after: datetime | None = None,
+            before: datetime | None = None,
+            guild_type: Literal["USER", "GUILD"] | None = None,
+            message_type: Literal["TextMESSAGE", "VoiceMESSAGE", "DirectMESSAGE"] | None = None,
+            sort_by: Literal["successful", "failed", "guild_snow", "guild_name", "author_snow", "author_name"] = "successful",
+            sort_by_direction: Literal["asc", "desc"] = "desc",
+            limit: int = 500,
+            group_by: Literal["year", "month", "day"] = "day"
+    ) -> list[tuple[date, int, int, int, str, int, str]]:
+        """
+
+        Parameters
+        -----------
+        guild: int
+            The snowflake id of the guild.
+        author: int
+            The snowflake id of the author.
+        after: datetime | None = None
+            Only count messages sent after the datetime.
+        before: datetime | None
+            Only count messages sent before the datetime.
+        guild_type: Literal["USER", "GUILD"] | None,
+            Type of guild.
+        message_type: Literal["TextMESSAGE", "VoiceMESSAGE", "DirectMESSAGE"] | None,
+            Type of message.
+        sort_by: Literal["successful", "failed", "guild_snow", "guild_name", "author_snow", "author_name"],
+            Sort items by selected.
+            Defaults to "successful"
+        sort_by_direction: Literal["asc", "desc"]
+            Sort items by ``sort_by`` in selected direction (asc = ascending, desc = descending).
+            Defaults to "desc"
+        limit: int = 500
+            Limit of the rows to return. Defaults to 500.
+        group_by: Literal["year", "month", "day"]
+            Results returned are grouped by ``group_by``
+
+        Returns
+        --------
+        list[tuple[date, int, int, int, str, int, str]]
+            List of tuples.
+
+            Each tuple contains:
+
+            - Date
+            - Successfule sends
+            - Failed sends
+            - Guild snowflake id,
+            - Guild name
+            - Author snowflake id,
+            - Author name
+
+        Raises
+        ------------
+        SQLAlchemyError
+            There was a problem with the database.
+        """
+        if after is None:
+            after = datetime.min
+
+        if before is None:
+            before = datetime.max
+
+        regions = ["day", "month", "year"]
+        regions = reversed(regions[regions.index(group_by):])
+        extract_stms = []
+        for region_ in regions:
+            extract_stms.append(func.extract(region_, MessageLOG.timestamp).cast(Integer))
+
+        async with self.session_maker() as session:
+            conditions = []
+            if guild is not None:
+                conditions.append(
+                    MessageLOG.guild.has(GuildUSER.snowflake_id == guild)
+                )
+
+            if author is not None:
+                conditions.append(
+                    MessageLOG.author.has(GuildUSER.snowflake_id == author)
+                )
+
+            if guild_type is not None:
+                conditions.append(
+                    MessageLOG.guild.has(
+                        GuildUSER.guild_type.has(GuildTYPE.name == guild_type)
+                    )
+                )
+
+            if message_type is not None:
+                conditions.append(
+                    MessageLOG.message_type.has(MessageTYPE.name == message_type)
+                )
+
+            count = (await self._run_async(
+                session.execute,
+                select(
+                    *extract_stms,
+                    func.sum(case((MessageLOG.success_rate > 0, 1), else_=0)).label("successful"),
+                    func.sum(case((MessageLOG.success_rate == 0, 1), else_=0)).label("failed"),
+                    select(GuildUSER.snowflake_id).where(GuildUSER.id == MessageLOG.guild_id).scalar_subquery().label("guild_snow"),
+                    select(GuildUSER.name).where(GuildUSER.id == MessageLOG.guild_id).scalar_subquery().label("guild_name"),
+                    select(GuildUSER.snowflake_id).where(GuildUSER.id == MessageLOG.author_id).scalar_subquery().label("author_snow"),
+                    select(GuildUSER.name).where(GuildUSER.id == MessageLOG.author_id).scalar_subquery().label("author_name")
+                )
+                .where(
+                    MessageLOG.timestamp.between(after, before),
+                    *conditions
+                )
+                .group_by(*extract_stms, MessageLOG.guild_id, MessageLOG.author_id)
+                .limit(limit)
+                .order_by(text(f"{sort_by} {sort_by_direction}"))
+            )).all()
+
+            extract_stm_len = len(extract_stms)
+            count = [
+                (date(*s[:extract_stm_len], *([1] * (3 - extract_stm_len))), *s[extract_stm_len:])
+                for s in count
+            ]
+
+            return count
+
+    async def analytic_get_message_log(
+            self,
+            guild: int | None = None,
+            author: int | None = None,
+            after: datetime | None = None,
+            before: datetime | None = None,
+            success_rate: tuple[float, float] = (0, 100),
+            guild_type: Literal["USER", "GUILD"] | None = None,
+            message_type: Literal["TextMESSAGE", "VoiceMESSAGE", "DirectMESSAGE"] | None = None,
+            sort_by: Literal["timestamp", "success_rate"] = "timestamp",
+            sort_by_direction: Literal["asc", "desc"] = "desc",
+            limit: int = 500,
+    ) -> list["MessageLOG"]:
+        """
+        Returns a list MessageLOG objects (message logs) that match the parameters.
+
+        Parameters
+        --------------
+        guild: int | None
+            The snowflake id of the guild.
+        author: int | None
+            The snowflake id of the author.
+        after: datetime | None
+            Include only messages sent after this datetime.
+        before: datetime | None
+            Include only messages sent before this datetime.
+        success_rate: tuple[int, int]
+            Success rate tuple containing minimum success rate and maximum success
+            rate the message log can have for it to be included.
+            Success rate is meassured in % and it is defined by:
+
+            Successuly sent channels / all channels.
+        guild_type: Literal["USER", "GUILD"] | None,
+            Type of guild.
+        message_type: Literal["TextMESSAGE", "VoiceMESSAGE", "DirectMESSAGE"] | None,
+            Type of message.
+        sort_by: Literal["timestamp", "success_rate", "data"],
+            Sort items by selected.
+            Defaults to "timestamp"
+        sort_by_direction: Literal["asc", "desc"] = "desc"
+            Sort items by ``sort_by`` in selected direction (asc = ascending, desc = descending).
+            Defaults to "desc"
+        limit: int = 500
+            Limit of the message logs to return. Defaults to 500.
+
+        Return
+        ---------
+        list[MessageLOG]
+            List of the message logs.
+        """
+        if after is None:
+            after = datetime.min
+
+        if before is None:
+            before = datetime.max
+
+        # Obtain internal guild id
+
+        async with self.session_maker() as session:
+            conditions = []
+            if guild is not None:
+                conditions.append(
+                    MessageLOG.guild.has(GuildUSER.snowflake_id == guild)
+                )
+
+            if author is not None:
+                conditions.append(
+                    MessageLOG.author.has(GuildUSER.snowflake_id == author)
+                )
+
+            if guild_type is not None:
+                conditions.append(
+                    MessageLOG.guild.has(
+                        GuildUSER.guild_type.has(GuildTYPE.name == guild_type)
+                    )
+                )
+
+            if message_type is not None:
+                conditions.append(
+                    MessageLOG.message_type.has(MessageTYPE.name == message_type)
+                )
+
+            messages = await self._run_async(
+                session.execute,
+                select(MessageLOG)
+                .where(
+                    MessageLOG.success_rate.between(*success_rate),
+                    MessageLOG.timestamp.between(after, before),
+                    *conditions
+                ).order_by(getattr(getattr(MessageLOG, sort_by), sort_by_direction)()).limit(limit)
+            )
+
+            return list(*zip(*messages.unique().all()))
+
     @misc._async_safe("safe_sem", 1)
     async def update(self, **kwargs):
         """
@@ -897,10 +1155,10 @@ if SQL_INSTALLED:
             Sequence("guild_user_seq", 0, 1, minvalue=0, maxvalue=2147483647, cycle=True),
             primary_key=True
         )
-        snowflake_id = mapped_column(BigInteger)
+        snowflake_id = mapped_column(BigInteger, index=True)
         name = mapped_column(String(3072))
         guild_type_id: Mapped[int] = mapped_column(ForeignKey("GuildTYPE.id"))
-        guild_type: Mapped["GuildTYPE"] = relationship(primaryjoin="GuildTYPE.id == GuildUSER.guild_type_id")
+        guild_type: Mapped["GuildTYPE"] = relationship(lazy="joined")
 
         def __init__(self,
                      guild_type: GuildTYPE,
@@ -932,7 +1190,7 @@ if SQL_INSTALLED:
         snowflake_id = mapped_column(BigInteger)
         name = mapped_column(String(3072))
         guild_id: Mapped[int] = mapped_column(ForeignKey("GuildUSER.id"))
-        guild: Mapped["GuildUSER"] = relationship()
+        guild: Mapped["GuildUSER"] = relationship(lazy="joined")
 
         def __init__(self,
                      snowflake: int,
@@ -966,62 +1224,6 @@ if SQL_INSTALLED:
                      content: dict):
             self.content = content
 
-    class MessageLOG(ORMBase):
-        """
-        Table containing information for each message send attempt.
-
-        NOTE: This table is missing successful and failed channels (or DM success status).
-        That is because those are a separate table.
-
-        Parameters
-        ------------
-        sent_data: str
-            JSONized data that was sent by the xMESSAGE object.
-        message_type: int
-            Foreign key  pointing to a row inside the MessageTYPE lookup table.
-        message_mode: int
-            Foreign key pointing to a row inside the MessageMODE lookup table.
-        dm_reason: str
-            If DM sent succeeded, it is null, if it failed it contains a string description of the error.
-        guild_id: int
-            Foreign key pointing the guild this message was advertised to.
-        """
-
-        __tablename__ = "MessageLOG"
-
-        id = mapped_column(
-            Integer,
-            Sequence("ml_seq", 0, 1, minvalue=0, maxvalue=2147483647, cycle=True),
-            primary_key=True
-        )
-        sent_data_id: Mapped[int] = mapped_column(ForeignKey("DataHISTORY.id"))
-        message_type_id: Mapped[int] = mapped_column(ForeignKey("MessageTYPE.id"))
-        guild_id: Mapped[int] = mapped_column(ForeignKey("GuildUSER.id"))
-        message_mode_id: Mapped[int] = mapped_column(ForeignKey("MessageMODE.id"), nullable=True)  # [TextMESSAGE, DirectMESSAGE]
-        dm_reason = mapped_column(String(3072))  # [DirectMESSAGE]
-        timestamp = mapped_column(DateTime)
-
-        sent_data: Mapped["DataHISTORY"] = relationship()
-        message_type: Mapped["MessageTYPE"] = relationship()
-        guild: Mapped["GuildUSER"] = relationship()
-        message_mode: Mapped["MessageMODE"] = relationship()
-        channels: Mapped[List["MessageChannelLOG"]] = relationship(back_populates="log")
-
-        def __init__(self,
-                     sent_data: DataHISTORY = None,
-                     message_type: MessageTYPE = None,
-                     message_mode: MessageMODE = None,
-                     dm_reason: str = None,
-                     guild: GuildUSER = None,
-                     channels: List["MessageChannelLOG"] = []):
-            self.sent_data = sent_data
-            self.message_type = message_type
-            self.message_mode = message_mode
-            self.dm_reason = dm_reason
-            self.guild = guild
-            self.timestamp = datetime.now().replace(microsecond=0)
-            self.channels = channels
-
     class MessageChannelLOG(ORMBase):
         """
         This is a table that contains a log of channels that are linked to a certain message log.
@@ -1042,11 +1244,89 @@ if SQL_INSTALLED:
         channel_id: Mapped[int] = mapped_column(Integer, ForeignKey("CHANNEL.id"), primary_key=True)
         reason = mapped_column(String(3072))
 
-        log: Mapped["MessageLOG"] = relationship(back_populates="channels", uselist=False)
-        channel: Mapped["CHANNEL"] = relationship()
+        log: Mapped["MessageLOG"] = relationship(back_populates="channels", uselist=False, lazy="joined")
+        channel: Mapped["CHANNEL"] = relationship(lazy="joined")
 
         def __init__(self,
                      channel: CHANNEL,
                      reason: str = None):
             self.channel = channel
             self.reason = reason
+
+    class MessageLOG(ORMBase):
+        """
+        Table containing information for each message send attempt.
+
+        NOTE: This table is missing successful and failed channels (or DM success status).
+        That is because those are a separate table.
+
+        Parameters
+        ------------
+        sent_data: DataHISTORY
+            DataHISTORY object containing JSON data.
+        message_type: MessageTYPE
+            MessageTYPE object representing type of the message.
+        message_mode: MessageMODE | None
+            MessageMODE object representing mode of the message. (TextMESSAGE and DirectMESSAGE only)
+        dm_reason: str | None
+            The reason why sending to the USER failed. (DirectMESSAGE only)
+        guild: GuildUSER
+            The guild / user message was sent to.
+        author: GuildUSER
+            The author of the message.
+        channels: List["MessageChannelLOG"]
+            List of MessageChannelLOG representing channel the message was sent into and the fail reason.
+        """
+
+        __tablename__ = "MessageLOG"
+
+        id = mapped_column(
+            Integer,
+            Sequence("ml_seq", 0, 1, minvalue=0, maxvalue=2147483647, cycle=True),
+            primary_key=True
+        )
+        sent_data_id: Mapped[int] = mapped_column(ForeignKey("DataHISTORY.id"))
+        message_type_id: Mapped[int] = mapped_column(ForeignKey("MessageTYPE.id"))
+        guild_id: Mapped[int] = mapped_column(ForeignKey("GuildUSER.id"))
+        author_id: Mapped[int] = mapped_column(ForeignKey("GuildUSER.id"))
+        # [TextMESSAGE, DirectMESSAGE]
+        message_mode_id: Mapped[int] = mapped_column(ForeignKey("MessageMODE.id"), nullable=True)
+        dm_reason = mapped_column(String(3072))  # [DirectMESSAGE]
+        timestamp = mapped_column(DateTime)
+
+        sent_data: Mapped["DataHISTORY"] = relationship(lazy="joined")
+        message_type: Mapped["MessageTYPE"] = relationship(lazy="joined")
+        guild: Mapped["GuildUSER"] = relationship(foreign_keys=[guild_id], lazy="joined")
+        author: Mapped["GuildUSER"] = relationship(foreign_keys=[author_id], lazy="joined")
+        message_mode: Mapped["MessageMODE"] = relationship(lazy="joined")
+        channels: Mapped[List["MessageChannelLOG"]] = relationship(back_populates="log", lazy="joined")
+        success_rate = column_property(
+            case(
+                (
+                    select(1).where(MessageChannelLOG.log_id == id).limit(1).exists(),
+                    100 * select(func.count()).where(MessageChannelLOG.reason.is_(None), MessageChannelLOG.log_id == id)
+                    .select_from(MessageChannelLOG)
+                    .scalar_subquery() /
+                    select(func.count()).where(MessageChannelLOG.log_id == id).select_from(MessageChannelLOG)
+                    .scalar_subquery(),
+                ),
+                else_=case((dm_reason.is_(None), 100), else_=0)
+            )
+        )
+
+        def __init__(self,
+                     sent_data: DataHISTORY = None,
+                     message_type: MessageTYPE = None,
+                     message_mode: MessageMODE = None,
+                     dm_reason: str = None,
+                     guild: GuildUSER = None,
+                     author: GuildUSER = None,
+                     channels: List["MessageChannelLOG"] = None):
+            self.sent_data = sent_data
+            self.message_type = message_type
+            self.message_mode = message_mode
+            self.dm_reason = dm_reason
+            self.guild = guild
+            self.author = author
+            self.timestamp = datetime.now()
+            self.channels = channels
